package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.urii.FileExpiry;
import org.netkernel.layer1.representation.FileRepresentation;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import java.io.File;
import java.io.FileOutputStream;
import java.net.URI;
import java.net.URLConnection;

public class FileSchemeAccessor extends StandardAccessorImpl
{
	private static final long DEFAULT_CACHE_VALID_TIME_MS = 1000l;
	public FileSchemeAccessor()
	{
		this.declareThreadSafe();
	}

	@Override
	public void onDelete(INKFRequestContext aContext) throws Exception
	{
		File f = getFile(aContext.getThisRequest().getArgumentValue("file"));
		boolean result = f.delete();
		INKFResponse resp = aContext.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

	@Override
	public void onExists(INKFRequestContext helper) throws Exception
	{
		File f = getFile(helper.getThisRequest().getArgumentValue("file"));
		boolean result = f.exists();
		INKFResponse resp = helper.createResponseFrom(result);
		if (result)      
		{
			long pollPeriod = getKernel().getConfiguration().getLong("netkernel.poll", DEFAULT_CACHE_VALID_TIME_MS);
			resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, new FileExpiry(f, pollPeriod));
		}
		else
		{
			resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
	}

	@Override
	public void onNew(INKFRequestContext helper) throws Exception
	{
		String pathAndFilename = helper.getThisRequest().getArgumentValue("file");
		File f = getFile(pathAndFilename);
		boolean result;
		if (pathAndFilename.endsWith("/"))
		{
			result = f.mkdirs();
		}
		else
		{
			File parent = new File(f.getParent());
			parent.mkdirs();
			result = f.createNewFile();
		}
		INKFResponse resp = helper.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

	@Override
	public void onSink(INKFRequestContext helper) throws Exception
	{
		IBinaryStreamRepresentation ibs = helper.sourcePrimary(IBinaryStreamRepresentation.class);
		File f = getFile(helper.getThisRequest().getArgumentValue("file"));
		if (f.isDirectory())
		{
			throw new java.io.FileNotFoundException("Cannot SINK to a directory");
		}
		else if (!f.exists())
		{
			File parent = new File(f.getParent());
			parent.mkdirs();
			f.createNewFile();
		}
		FileOutputStream fos = new FileOutputStream(f);
		try
		{	ibs.write(fos);
		}
		finally
		{	fos.close();			
		}
		INKFResponse resp = helper.createResponseFrom(null);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

	@Override
	public void onSource(INKFRequestContext helper) throws Exception
	{
		String furi = helper.getThisRequest().getArgumentValue("file");
		Object rep = null;
		URI uri=URI.create(furi);
		File f = new File(uri);
		if (!f.exists())
		{
			rep = new java.io.FileNotFoundException("File " + furi + " not found");
		}
		else
		{
			rep = new FileRepresentation(f);
		}
		INKFResponse resp = helper.createResponseFrom(rep);

		URLConnection c = uri.toURL().openConnection();
		String mime = c.getContentType();
		resp.setMimeType(mime);

		long pollPeriod = getKernel().getConfiguration().getLong("netkernel.poll", DEFAULT_CACHE_VALID_TIME_MS);
		if (f != null)
		{
			resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, new FileExpiry(f, pollPeriod));
		}
	}

	private File getFile(String aURI)
	{
		return new File(URI.create(aURI));
	}


}
